@@ -1,6 +1,6 @@
 """SourceString stores the entire string to be parsed in memory and provides
-some simple methods for retrieving and moving current position aswell as methods
-for matching strings and patterns.
+some simple methods for retrieving and moving current position aswell as
+methods for matching strings and patterns.
 """
 __author__ = 'Taylor "Nekroze" Lawson'
 __email__ = 'nekroze@eturnilnetwork.com'
@@ -12,21 +12,23 @@ class SourceString(object):
 
     It also stores the current row and column position as manually counted.
 
-    Provides multiple methods for matching strings and patterns and working with
-    the source string.
+    Provides multiple methods for matching strings and patterns and working
+    with the source string.
     """
-    def __init__(self, string = None):
+    def __init__(self, string=None):
         """Accepts a string or None by default. If a string is given then
-        self.set_string(string) is run automatically. If you wish to load a file
-        then create a SourceString object with no arguments and then use
-        load_file or overload this function when inheriting from SourceString
+        self.set_string(string) is run automatically. If you wish to load a
+        file then create a SourceString object with no arguments and then use
+        load_file or overload this function when inheriting from SourceString.
         """
         self.string = ''
         self.length = 0
+        self.pos = 0
+        self.col = 0
+        self.row = 1
+        self.eos = 0
         if string is not None:
             self.set_string(string)
-        else:
-            self.reset_position()
 
     def load_file(self, filename):
         """Read in file contents and set the current string."""
@@ -52,12 +54,51 @@ class SourceString(object):
         self.row = 1
         self.eos = 0
 
-    def has_space(self, length = 1):
+    def has_space(self, length=1):
         """Returns boolean if self.pos + length < working string length."""
         return self.pos + length - 1 < self.length
 
+    def eol_distance_next(self):
+        """Return the ammount of characters until the next newline."""
+        distance = 0
+        for char in self.string[self.pos:]:
+            if char == '\n':
+                break
+            else:
+                distance += 1
+        return distance
+
+    def eol_distance_last(self):
+        """Return the ammount of characters until the last newline."""
+        distance = 0
+        for char in reversed(self.string[:self.pos]):
+            if char == '\n':
+                break
+            else:
+                distance += 1
+        return distance
+
+    def spew_length(self, length):
+        """Move current position backwards by length."""
+        pos = self.pos
+        if not pos or length > pos:
+            return None
+
+        row = self.row
+        for char in reversed(self.string[pos - length:pos]):
+            pos -= 1
+            if char == '\n':  # handle a newline char
+                row -= 1
+
+        self.pos = pos
+        self.col = self.eol_distance_last()
+        self.row = row
+
+        if self.has_space():  # Set eos if there is no more space left.
+            self.eos = 0
+
     def eat_length(self, length):
-        """Move current position by length and set eos if not has_space()."""
+        """Move current position forward by length and sets eos if needed."""
         pos = self.pos
         if self.eos or pos + length > self.length:
             return None
@@ -118,7 +159,7 @@ class SourceString(object):
 
         return self.string[self.pos]
 
-    def get_length(self, length, trim = 0):
+    def get_length(self, length, trim=0):
         """Return string at current position + length.
         If trim == true then get as much as possible before eos.
         """
@@ -144,7 +185,7 @@ class SourceString(object):
         else:
             return string[pos:]
 
-    def rest_of_string(self, offset = 0):
+    def rest_of_string(self, offset=0):
         """A copy of the current position till the end of the source string."""
         if self.has_space(offset):
             return self.string[self.pos + offset:]
@@ -170,12 +211,15 @@ class SourceString(object):
 
     def get_current_line(self):
         """Return a SourceLine of the current line."""
+        if not self.has_space():
+            return None
+        
         pos = self.pos - self.col
         string = self.string
         end = self.length
 
         output = []
-        while string[pos] != '\n':
+        while pos < len(string) and string[pos] != '\n':
             output.append(string[pos])
             pos += 1
             if pos == end:
@@ -189,7 +233,7 @@ class SourceString(object):
         return SourceLine(''.join(output), self.row)
 
     def get_lines(self, first, last):
-        """Return SourceLines for lines between and including first and last."""
+        """Return SourceLines for lines between and including first & last."""
         line = 1
         linestring = []
         linestrings = []
@@ -211,7 +255,7 @@ class SourceString(object):
 
         return [SourceLine(string, lineno) for string, lineno in linestrings]
 
-    def get_surrounding_lines(self, past = 1, future = 1):
+    def get_surrounding_lines(self, past=1, future=1):
         """Return the current line and x,y previous and future lines.
         Returns a list of SourceLine's.
         """
@@ -265,7 +309,7 @@ class SourceString(object):
 
         return output
 
-    def match_string(self, string, word = 0):
+    def match_string(self, string, word=0):
         """Returns 1 if string can be matches against SourceString's
         current position.
 
@@ -275,7 +319,7 @@ class SourceString(object):
             return self.get_string() == string
         return self.get_length(len(string)) == string
 
-    def match_any_string(self, strings, word = 0):
+    def match_any_string(self, strings, word=0):
         """Attempts to match each string in strings in order.
         Will return the string that matches or an empty string if no match.
 
@@ -288,8 +332,6 @@ class SourceString(object):
         if word:
             current = self.get_string()
             return current if current in strings else ''
-
-        #strings = sorted(strings, key = len)
         current = ''
 
         currentlength = 0
@@ -309,7 +351,7 @@ class SourceString(object):
         current = self.string[self.pos]
         return current if current in chars else ''
 
-    def match_string_pattern(self, first, rest = None, least = 1):
+    def match_string_pattern(self, first, rest=None, least=1):
         """Match each char sequentially from current SourceString position
         until the pattern doesnt match and return all maches.
 
@@ -337,7 +379,7 @@ class SourceString(object):
 
         return ''.join(output)
 
-    def match_function_pattern(self, first, rest = None, least = 1):
+    def match_function_pattern(self, first, rest=None, least=1):
         """Match each char sequentially from current SourceString position
         until the pattern doesnt match and return all maches.
 
@@ -369,7 +411,7 @@ class SourceString(object):
 
         return ''.join(output)
 
-    def count_indents(self, spacecount, tabs = 0):
+    def count_indents(self, spacecount, tabs=0):
         """Counts the number of indents that can be tabs or spacecount
         number of spaces in a row from the current position.
         """
@@ -388,7 +430,7 @@ class SourceString(object):
                 spaces = 0
         return indents
 
-    def count_indents_length(self, spacecount, tabs = 0):
+    def count_indents_length(self, spacecount, tabs=0):
         """Counts the number of indents that can be tabs or spacecount
         number of spaces in a row from the current position.
 
@@ -411,8 +453,11 @@ class SourceString(object):
                 spaces = 0
         return (indents, charlen)
 
-    def count_indents_last_line(self, spacecount, tabs = 0, back = 5):
-        """Finds the last meaningful line and returns its indent level."""
+    def count_indents_last_line(self, spacecount, tabs=0, back=5):
+        """Finds the last meaningful line and returns its indent level.
+        Back specifies the amount of lines to look back for a none whitespace
+        line.
+        """
         lines = self.get_surrounding_lines(back, 0)
 
         for line in reversed(lines):
@@ -420,9 +465,11 @@ class SourceString(object):
                 return line.count_indents(spacecount, tabs)
         return 0
 
-    def count_indents_length_last_line(self, spacecount, tabs = 0, back = 5):
+    def count_indents_length_last_line(self, spacecount, tabs=0, back=5):
         """Finds the last meaningful line and returns its indent level and
         character length.
+        Back specifies the amount of lines to look back for a none whitespace
+        line.
         """
         lines = self.get_surrounding_lines(back, 0)
 
@@ -431,7 +478,7 @@ class SourceString(object):
                 return line.count_indents_length(spacecount, tabs)
         return (0, 0)
 
-    def skip_whitespace(self, newlines = 0):
+    def skip_whitespace(self, newlines=0):
         """Moves the position forwards to the next non newline space character.
         If newlines >= 1 include newlines as spaces.
         """
@@ -483,6 +530,7 @@ class SourceString(object):
             yield string[self.pos]
             self.eat_length(1)
 
+
 class SourceLine(SourceString):
     """Contains an entire line of a source with handy line specific methods."""
 
@@ -508,8 +556,11 @@ class SourceLine(SourceString):
             if not char.isspace():
                 return char
 
-    def pretty_print(self):
-        """Return a string of this line including linenumber."""
+    def pretty_print(self, carrot=False):
+        """Return a string of this line including linenumber.
+        If carrot is True then a line is added under the string with a carrot
+        under the current character position.
+        """
         lineno = self.lineno
         padding = 0
         if lineno < 1000:
@@ -519,7 +570,10 @@ class SourceLine(SourceString):
         if lineno < 10:
             padding = 3
 
-        return str(lineno) + (' ' * padding) + '|' + self.string
+        string = str(lineno) + (' ' * padding) + '|' + self.string
+        if carrot:
+            string += '\n' + (' ' * (self.col + 5))
+        return string
 
     def __str__(self):
         return self.pretty_print()
